#include <PalmOS.h>
#include <MemoryMgr.h>
#include <DataMgr.h>

#include "PalmOSGUI.h"
#include "PalmHelper.h"
#include "FrodoResources.h"


//***********************************************************
// Procedure: ShowInfo
//
// Parameter: s  Text to display
//
// Returnval: -
//
// Description: Show informtaion to user. 
//***********************************************************
void ShowInfo(char *s)
{
	FrmCustomAlert(GeneralInfo, s, "", "");
}


//***********************************************************
// Procedure: ShowInfoRes
//
// Parameter: lID   ID of string resource with text
//
// Returnval: -
//
// Description: Display message from string resource. 
//***********************************************************
void ShowInfoRes(DmResID lID)
{
  MemPtr strMessage;
  MemHandle h = 0;

  h = GetResource(stringType, lID);
  if(h)
  {
    strMessage = MemHandleLock(h);
    FrmCustomAlert(GeneralInfo, strMessage, "", "");
    MemHandleUnlock(h);
    FreeResource(h);
  }
  else
    FrmCustomAlert(GeneralDebug, "Unable to find resource with info text.", "", "");
}


//***********************************************************
// Procedure: ShowDebug
//
// Parameter: s  Text to display
//
// Returnval: -
//
// Description: Show debug informtaion. 
//***********************************************************
void ShowDebug(char *s)
{
	FrmCustomAlert(GeneralDebug, s, "", "");
}


//***********************************************************
// Procedure: ShowError
//
// Parameter: strMessage  Error message to display
//
// Returnval: -
//
// Description: Display error message. 
//***********************************************************
void ShowError(char *strMessage)
{
  FrmCustomAlert(GeneralError, strMessage, "", "");
}


//***********************************************************
// Procedure: ShowErrorRes
//
// Parameter: lID   ID of string resource with error text
//
// Returnval: -
//
// Description: Display error message from string resource. 
//***********************************************************
void ShowErrorRes(DmResID lID)
{
  MemPtr strMessage;
  MemHandle h = 0;

  h = GetResource(stringType, lID);
  if(h)
  {
    strMessage = MemHandleLock(h);
    FrmCustomAlert(GeneralError, strMessage, "", "");
    MemHandleUnlock(h);
    FreeResource(h);
  }
  else
    FrmCustomAlert(GeneralDebug, "Unable to find resource with error text.", "", "");
}


//***********************************************************
// Procedure: ShowErrorResStr
//
// Parameter: lID       ID of string resource with error text
//            strParam  String to replace %s in error text
//
// Returnval: -
//
// Description: Display error message from string resource. 
//***********************************************************
void ShowErrorResStr(DmResID lID, char *strParam)
{
  MemPtr strMessage;
  MemHandle h = 0;
  char strTemp[256];
  
  h = GetResource(stringType, lID);
  if(h)
  {
    strMessage = MemHandleLock(h);
    StrPrintF(strTemp, strMessage, strParam);
    FrmCustomAlert(GeneralError, strTemp, "", "");
    MemHandleUnlock(h);
    FreeResource(h);
  }
  else
    FrmCustomAlert(GeneralDebug, "Unable to find resource with error text.", "", "");
}


//***********************************************************
// Procedure: ShowErrorResLong
//
// Parameter: lID       ID of string resource with error text
//            lParam    Value to replace %ld or %lx in error text
//
// Returnval: -
//
// Description: Display error message from string resource. 
//***********************************************************
void ShowErrorResLong(DmResID lID, int32 lParam)
{
  MemPtr strMessage;
  MemHandle h = 0;
  char strTemp[256];
  
  h = GetResource(stringType, lID);
  if(h)
  {
    strMessage = MemHandleLock(h);
    StrPrintF(strTemp, strMessage, lParam);
    FrmCustomAlert(GeneralError, strTemp, "", "");
    MemHandleUnlock(h);
    FreeResource(h);
  }
  else
    FrmCustomAlert(GeneralDebug, "Unable to find resource with error text.", "", "");
}


//***********************************************************
// Procedure: AskUserDialog
//
// Parameter: strQuestion   Question to ask
//
// Returnval: 0 -> user selected first button (yes), 
//            1 -> user selected second button (no)
//
// Description: Ask user
//***********************************************************
UInt16 AskUserDialog(char *strQuestion)
{
  if(FrmCustomAlert(AskUserAlert, strQuestion, "", ""))
    return 0; // User selected first button -> Yes
  else
    return 1; // User selected second button -> No
}


//***********************************************************
// Procedure: NewSwapped
//
// Parameter: n     Number of bytes
//
// Returnval: Pointer to locked memory (in armlet endian!) or 0 on error
//
// Description: Allocate memory and lock pointer. 
//***********************************************************
MemPtr NewSwapped(UInt32 n)
{
	MemPtr p=MemPtrNew(n);
	return (MemPtr) swap32((UInt32)p);
}


//***********************************************************
// Procedure: FreeSwapped
//
// Parameter: p     Pointer to memory (in armlet endian!)
//
// Returnval: -
//
// Description: Free locked memory. 
//***********************************************************
void FreeSwapped(MemPtr p)
{
	MemPtr p2=(MemPtr) swap32((UInt32)p);
	if (p2) MemPtrFree(p2);
}


//***********************************************************
// Procedure: GetRes
//
// Parameter: t     Resource type
//            idx   Index of resource
//            name  Name of resource
//
// Returnval: Pointer to locked resource (in armlet endian!) or 0 on error
//
// Description: Load resource and lock memory pointer. 
//***********************************************************
MemPtr GetRes(DmResType t, UInt32 idx, char *name)
{
	MemHandle h;
	MemPtr p;

	h = GetResource(t, idx);
	if (!h)
	{
		ShowErrorResStr(StringErrResourceNotFound, name);
		return NULL;
	}
	p=MemHandleLock(h);
	if (!p) 
	{
		FreeResource(h);
		return NULL;
	}
	return (MemPtr) swap32((UInt32)p);
}


//***********************************************************
// Procedure: UngetRes
//
// Parameter: p     Pointer to memory of locked resource (in armlet endian!)
//
// Returnval: -
//
// Description: Free locked resource. 
//***********************************************************
void UngetRes(MemPtr p)
{
	MemHandle h;
	MemPtr p2;

	if (!p) 
		return;

	p2=(MemPtr) swap32((UInt32)p);

	h=MemPtrRecoverHandle(p2);
	if (!h) 
	{
		if (!exiting) 
			ShowErrorResLong(StringErrRecoverFailed, (int32) p2);
		return;
	}
	MemHandleUnlock(h);
	FreeResource(h);
}


//***********************************************************
// Procedure: CopyRes
//
// Parameter: t     Resource type
//            idx   Index of resource
//            name  Name of resource
//
// Returnval: Pointer to locked memory (in armlet endian!) or 0 on error
//
// Description: Load resource into writeable memory and lock pointer. 
//              (Quiet version)
//***********************************************************
MemPtr CopyRes(DmResType t, UInt32 idx, char *name)
{
	return CopyRes2(t, idx, name, 0);
}


//***********************************************************
// Procedure: CopyRes2
//
// Parameter: t     Resource type
//            idx   Index of resource
//            name  Name of resource
//            quiet Flag if error messages should be displayed
//
// Returnval: Pointer to locked memory (in armlet endian!) or 0 on error
//
// Description: Load resource into writeable memory and lock pointer. 
//***********************************************************
MemPtr CopyRes2(DmResType t, UInt32 idx, char *name, int quiet)
{
	MemHandle h;
	MemPtr src, dst;

	h = GetResource(t, idx);
	if (!h) 
	{
		if (!quiet)
			ShowErrorResStr(StringErrResourceNotFound, name);
		return NULL;
	}
	// Allocate memory, copy resource
	dst = MemPtrNew(MemHandleSize(h));
	if (!dst) 
	{
		FreeResource(h);
		if (!quiet)
		  ShowErrorResStr(StringErrOutOfMemoryLoading, name);
		return NULL;
	}
	src = MemHandleLock(h);
	MemMove(dst, src, MemHandleSize(h));
	MemHandleUnlock(h);
	FreeResource(h);

	return (MemPtr) swap32((UInt32)dst);
}


//***********************************************************
// Procedure: UncopyRes
//
// Parameter: p     Pointer to memory (in armlet endian!)
//
// Returnval: -
//
// Description: Free locked memory of copied resource. 
//***********************************************************
void UncopyRes(void *p)
{
	MemPtr p2=(MemPtr) swap32((UInt32)p);
	if (p2) 
		MemPtrFree(p2);
}


//***********************************************************
// Procedure: LoadRes
//
// Parameter: t     Resource type
//            idx   Index of resource
//            name  Name of resource
//
// Returnval: Pointer to locked memory or 0 on error
//
// Description: Load resource into writeable memory and lock pointer. 
//***********************************************************
MemPtr LoadRes(DmResType t, UInt32 idx, char *name)
{
	MemHandle h;
	MemPtr src, dst;
  UInt32 lSize;
  
	h = GetResource(t, idx);
	if (!h) 
	{
		ShowErrorResStr(StringErrResourceNotFound, name);
		return NULL;
	}
	// Allocate memory, copy resource
  lSize = MemHandleSize(h);
  if(!lSize)
  {
		FreeResource(h);
		return NULL;
  }
	dst = MemPtrNew(lSize);
	if (!dst) 
	{
		FreeResource(h);
	  ShowErrorResStr(StringErrOutOfMemoryLoading, name);
		return NULL;
	}
	src = MemHandleLock(h);
	if(src)
	{
  	MemMove(dst, src, lSize);
  	MemHandleUnlock(h);
  }
	FreeResource(h);

	return dst;
}
